==> em_tasks-0.1.64-py2.py3-none-any.whl: em_tasks/tools/amz_uploadable_products_formatter.py <==
# -*- coding: utf-8 -*-

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.utils.pipelines.price_calculator import PriceCalculatorPipeline
from em_tasks.utils.pipelines.image_url_corrector import ImageUrlCorrectorPipeline
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.utils.pipelines.upc_validator import UpcValidatorPipeline
from em_tasks.store.models import *


class AmzUploadableProductsFormatter():
  def __init__(self, store_code, marketplace, product_service, offer_service, price_calculator, output_path):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return

    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.blacklist_filter = BlacklistFilter()

    self.product_service = product_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.pc = AmzProductConverter(self.marketplace)
    self.indice_name = 'amz_products_api_{}'.format(marketplace.lower())
    self.condition = 'new'
    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')

    self.sales_rank_min = 500000
    self.weight_max = 2.5
    self.dimension_max = 20
    if self.marketplace == 'us':
      self.dimension_max = 30
    elif self.marketplace == 'uk':
      self.sales_rank_min = 1500000
      self.weight_max = 2
      self.dimension_max = 12
    else:
      self.sales_rank_min = 1500000
      self.dimension_max = 20

    self.blacklisted_path = os.path.abspath(os.path.join(os.path.expanduser('~'), '.em_tasks', 'amz_{}'.format(self.marketplace), 'blacklisted_products.txt'))
    blacklisted_dir = os.path.dirname(self.blacklisted_path)
    if not os.path.isdir(blacklisted_dir):
      os.makedirs(blacklisted_dir)

    self.blacklisted_products = dict()

    self.unuploadable_path = os.path.abspath(os.path.join(os.path.expanduser('~'), '.em_tasks', 'amz_{}'.format(self.marketplace), 'unuploadable_products.txt'))
    unuploadable_dir = os.path.dirname(self.unuploadable_path)
    if not os.path.isdir(unuploadable_dir):
      os.makedirs(unuploadable_dir)

    self.unuploadable_products = dict()

    self.products_filter_pipeline = AmzProductsFilterPipeline(self.marketplace, self.weight_max, self.dimension_max, self.sales_rank_min)
    price_calculator_pipeline = PriceCalculatorPipeline(price_calculator)
    image_url_corrector_pipeline = ImageUrlCorrectorPipeline()
    upc_validator_pipeline = UpcValidatorPipeline()
    self.pipelines = [price_calculator_pipeline, image_url_corrector_pipeline, upc_validator_pipeline]

  def run(self):
    try:
      self.load_blacklisted_products()

      logger.debug('[BlacklistedProductsLoaded] ProductsCnt: %s', len(self.blacklisted_products))
    except Exception as e:
      logger.exception(e)

    try:
      self.load_unuploadable_products()

      logger.debug('[UnuploadableProductsLoaded] ProductsCnt: %s', len(self.unuploadable_products))
    except Exception as e:
      logger.exception(e)

    try:
      self.uploaded_prods = dict()
      for prod in self.product_util.get_amz_products(self.marketplace):
        source_product_id = prod.get('source_product_id', prod.get('asin', None))
        if not source_product_id:
          continue

        self.uploaded_prods[source_product_id] = None

      logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(self.uploaded_prods))
    except Exception as e:
      logger.exception(e)

    try:
      prods_buf = dict()
      batch_size = 500
      for prod_id, prod in self.product_service.load_products(self.indice_name):
        # exist = prod.get('existence', True)
        # if not exist:
        #   continue

        # has_only_default_variant = prod.get('has_only_default_variant', True)
        # if not has_only_default_variant:
        #   continue

        # Filter blacklisted products and already uploaded products
        if prod_id in self.blacklisted_products:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', prod_id, self.blacklisted_products[prod_id])
          continue

        if prod_id in self.unuploadable_products:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', prod_id, self.unuploadable_products[prod_id])
          continue

        if prod_id in self.uploaded_prods:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Uploaded - %s', prod_id, self.uploaded_prods[prod_id])
          continue

        result = self.products_filter_pipeline.check_product(prod)
        passed = result.get('passed', True)
        reason = result.get('reason', '')
        if not passed:
          self.unuploadable_products[prod_id] = reason
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', prod_id, reason)
          continue

        result = self.check_blacklist(prod)
        passed = result.get('passed', True)
        reason = result.get('reason', '')
        if not passed:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', prod_id, reason)
          self.blacklisted_products[prod_id] = reason
          continue

        prods_buf[prod_id] = prod
        if len(prods_buf) < batch_size:
          continue

        self.process_products(prods_buf)
        prods_buf.clear()

      if prods_buf:
        self.process_products(prods_buf)
    finally:
      self.output_fh.close()

      self.save_blacklisted_products()
      self.save_unuploadable_products()

  def process_products(self, prods):
    asins = list(prods.keys())

    offers = dict()
    src_offers = None
    max_retries = 3
    while max_retries > 0:
      try:
        src_offers = self.offer_service.get_offers(self.marketplace, asins, self.condition)
        if not isinstance(src_offers, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)
    if not src_offers:
      return

    products = []
    for asin, product_info in prods.items():
      prod = self.pc.convert(product_info)

      src_offer = src_offers.get(asin, None)
      if not src_offer:
        logger.debug('[NoOffer] ASIN: %s', asin)
        continue

      prod['price'] = src_offer.get('price', 0)
      prod['currency'] = src_offer.get('currency', 'USD')
      products.append(prod)

    for prod in products:
      product = prod
      for pipeline in self.pipelines:
        product = pipeline.execute(product)
        if not product:
          break

      if not product or not product.get('price', 0):
        continue

      product["shipping_days_min"] = None
      product["shipping_days_max"] = None
      self.output_fh.write(json.dumps(product, ensure_ascii=False))
      self.output_fh.write("\n")

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}
    s = '{}-{}'.format(product.get('title', '').lower(), self.pc.get_brand(product).lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted

  def load_blacklisted_products(self):
    if not os.path.isfile(self.blacklisted_path):
      return

    with open(self.blacklisted_path, encoding='utf-8', errors='ignore') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        parts = s.split(",")
        pid = parts[0]
        self.blacklisted_products[pid] = parts[-1]

  def save_blacklisted_products(self):
    if not self.blacklisted_products:
      return

    records = []
    with open(self.blacklisted_path, 'w+', encoding='utf-8', errors='ignore') as fh:
      for pid, reason in self.blacklisted_products.items():
        records.append('{},{}'.format(pid, reason))

      fh.write("\n".join(records))
      fh.write("\n")

  def load_unuploadable_products(self):
    if not os.path.isfile(self.unuploadable_path):
      return

    with open(self.unuploadable_path, encoding='utf-8', errors='ignore') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        parts = s.split(",")
        pid = parts[0]
        self.unuploadable_products[pid] = parts[-1]

  def save_unuploadable_products(self):
    if not self.unuploadable_products:
      return

    records = []
    with open(self.unuploadable_path, 'w+', encoding='utf-8', errors='ignore') as fh:
      for pid, reason in self.unuploadable_products.items():
        records.append('{},{}'.format(pid, reason))

      fh.write("\n".join(records))
      fh.write("\n")

==> em_tasks-0.1.65-py2.py3-none-any.whl: em_tasks/tools/amz_uploadable_products_formatter.py <==
# -*- coding: utf-8 -*-

import datetime
import time

import dateparser

from em_tasks import logger
from em_tasks.utils.product_service import ProductService
from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil
from em_tasks.utils.product_converters import AmzProductConverter
from em_tasks.utils.blacklist_filter import BlacklistFilter
from em_tasks.utils.pipelines.price_calculator import PriceCalculatorPipeline
from em_tasks.utils.pipelines.price_filter import PriceFilterPipeline
from em_tasks.utils.pipelines.image_url_corrector import ImageUrlCorrectorPipeline
from em_tasks.utils.pipelines.amz_products_filter import AmzProductsFilterPipeline
from em_tasks.utils.pipelines.upc_validator import UpcValidatorPipeline
from em_tasks.store.models import *
from em_tasks.utils.product_loaders.amz_es_product_loader import AmzEsProductLoader



class AmzUploadableProductsFormatter():
  def __init__(self, store_code, marketplace, product_service, offer_service, price_calculator, output_path, products_path, source, source_code, sku_prefix='', ttl=7):
    self.store = StoreUtil.get_store_by_code(store_code)
    if not self.store:
      return
    self.products_path = products_path
    credential = json.loads(self.store.api_credential)
    self.product_util = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))

    self.blacklist_filter = BlacklistFilter()

    self.product_service = product_service
    self.offer_service = offer_service
    self.marketplace = marketplace.lower()
    self.pc = AmzProductConverter(self.marketplace)
    self.indice_name = 'amz_products_api_{}'.format(marketplace.lower())
    self.condition = 'new'
    self.output_path = os.path.abspath(os.path.expanduser(output_path))
    output_dir = os.path.dirname(self.output_path)
    if not os.path.isdir(output_dir):
      os.makedirs(output_dir)
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')

    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl)
    self.sales_rank_min = 1000000
    self.weight_max = 2.5
    self.dimension_max = 20
    if self.marketplace == 'us':
      self.dimension_max = 30
    elif self.marketplace == 'uk':
      self.sales_rank_min = 1500000
      self.weight_max = 2
      self.dimension_max = 12
    else:
      self.sales_rank_min = 1500000
      self.dimension_max = 20

    self.blacklisted_path = os.path.abspath(os.path.join(os.path.expanduser('~'), '.em_tasks', 'amz_{}'.format(self.marketplace), 'blacklisted_products.txt'))
    blacklisted_dir = os.path.dirname(self.blacklisted_path)
    if not os.path.isdir(blacklisted_dir):
      os.makedirs(blacklisted_dir)

    self.blacklisted_products = dict()

    self.unuploadable_path = os.path.abspath(os.path.join(os.path.expanduser('~'), '.em_tasks', 'amz_{}'.format(self.marketplace), 'unuploadable_products.txt'))
    unuploadable_dir = os.path.dirname(self.unuploadable_path)
    if not os.path.isdir(unuploadable_dir):
      os.makedirs(unuploadable_dir)

    self.unuploadable_products = dict()

    self.products_filter_pipeline = AmzProductsFilterPipeline(self.marketplace, self.weight_max, self.dimension_max, self.sales_rank_min)
    price_calculator_pipeline = PriceCalculatorPipeline(price_calculator)
    price_filter_pipeline = PriceFilterPipeline()
    image_url_corrector_pipeline = ImageUrlCorrectorPipeline()
    upc_validator_pipeline = UpcValidatorPipeline()
    self.pipelines = [price_calculator_pipeline, price_filter_pipeline, image_url_corrector_pipeline, upc_validator_pipeline]

  def run(self):
    try:
      self.load_blacklisted_products()

      logger.debug('[BlacklistedProductsLoaded] ProductsCnt: %s', len(self.blacklisted_products))
    except Exception as e:
      logger.exception(e)

    try:
      self.load_unuploadable_products()

      logger.debug('[UnuploadableProductsLoaded] ProductsCnt: %s', len(self.unuploadable_products))
    except Exception as e:
      logger.exception(e)

    try:
      self.uploaded_prods = dict()
      for prod in self.product_util.get_amz_products(self.marketplace):
        source_product_id = prod.get('source_product_id', prod.get('asin', None))
        if not source_product_id:
          continue

        self.uploaded_prods[source_product_id] = None

      logger.debug('[UploadedProductsLoaded] ProductsCnt: %s', len(self.uploaded_prods))
    except Exception as e:
      logger.exception(e)

    try:
      prods_buf = dict()
      batch_size = 500
      products_loader = AmzEsProductLoader(self.product_service, self.offer_service, self.marketplace, self.products_path)
      for prod_id, prod in products_loader.load_products_by_id():
        # exist = prod.get('existence', True)
        # if not exist:
        #   continue

        # has_only_default_variant = prod.get('has_only_default_variant', True)
        # if not has_only_default_variant:
        #   continue

        # Filter blacklisted products and already uploaded products
        if prod_id in self.blacklisted_products:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', prod_id, self.blacklisted_products[prod_id])
          continue

        if prod_id in self.unuploadable_products:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', prod_id, self.unuploadable_products[prod_id])
          continue

        if prod_id in self.uploaded_prods:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Uploaded - %s', prod_id, self.uploaded_prods[prod_id])
          continue

        result = self.products_filter_pipeline.check_product(prod)
        passed = result.get('passed', True)
        reason = result.get('reason', '')
        if not passed:
          self.unuploadable_products[prod_id] = reason
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Unuploadable - %s', prod_id, reason)
          continue

        result = self.check_blacklist(prod)
        passed = result.get('passed', True)
        reason = result.get('reason', '')
        if not passed:
          logger.debug('[ProductFiltered] ASIN: %s, Reason: Blacklisted - %s', prod_id, reason)
          self.blacklisted_products[prod_id] = reason
          continue

        prods_buf[prod_id] = prod
        if len(prods_buf) < batch_size:
          continue

        self.process_products(prods_buf)
        prods_buf.clear()

      if prods_buf:
        self.process_products(prods_buf)
    finally:
      self.output_fh.close()

      self.save_blacklisted_products()
      self.save_unuploadable_products()

  def process_products(self, prods):
    asins = list(prods.keys())

    offers = dict()
    src_offers = None
    max_retries = 3
    while max_retries > 0:
      try:
        src_offers = self.offer_service.get_offers(self.marketplace, asins, self.condition)
        if not isinstance(src_offers, dict):
          logger.info('[OfferService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)
    if not src_offers:
      return

    products = []
    for asin, product_info in prods.items():
      prod = self.pc.convert(product_info)

      src_offer = src_offers.get(asin, None)
      if not src_offer:
        logger.debug('[NoOffer] ASIN: %s', asin)
        continue

      offer_time_s = src_offer.get('time', None)
      if offer_time_s:
        offer_time = None
        try:
          offer_time = dateparser.parse(offer_time_s).date()
        except Exception as e:
          logger.exception(e)

        if offer_time:
          if offer_time < self.expire_date:
            logger.debug('[OfferExpired] ASIN: %s, Offer: %s', asin, src_offer)
            prod['price'] = 0
            prod['currency'] = src_offer.get('currency', 'USD')
          else:
            prod['price'] = src_offer.get('price', 0)
            prod['currency'] = src_offer.get('currency', 'USD')
        else:
          prod['price'] = 0
          prod['currency'] = src_offer.get('currency', 'USD')
      else:
        prod['price'] = src_offer.get('price', 0)
        prod['currency'] = src_offer.get('currency', 'USD')
      products.append(prod)

    for prod in products:
      product = prod
      for pipeline in self.pipelines:
        product = pipeline.execute(product)
        if not product:
          break

      if not product or not product.get('price', 0):
        continue

      product["shipping_days_min"] = None
      product["shipping_days_max"] = None
      self.output_fh.write(json.dumps(product, ensure_ascii=False))
      self.output_fh.write("\n")

  def check_blacklist(self, product):
    blacklisted = {'passed': True, 'reason': ''}
    s = '{}-{}'.format(product.get('title', '').lower(), self.pc.get_brand(product).lower())
    result = self.blacklist_filter.is_blacklisted(s)
    if result:
      blacklisted = {'passed': False, 'reason': '[BlacklistKeyword] {}'.format(result)}

    return blacklisted

  def load_blacklisted_products(self):
    if not os.path.isfile(self.blacklisted_path):
      return

    with open(self.blacklisted_path, encoding='utf-8', errors='ignore') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        parts = s.split(",")
        pid = parts[0]
        self.blacklisted_products[pid] = parts[-1]

  def save_blacklisted_products(self):
    if not self.blacklisted_products:
      return

    records = []
    with open(self.blacklisted_path, 'w+', encoding='utf-8', errors='ignore') as fh:
      for pid, reason in self.blacklisted_products.items():
        records.append('{},{}'.format(pid, reason))

      fh.write("\n".join(records))
      fh.write("\n")

  def load_unuploadable_products(self):
    if not os.path.isfile(self.unuploadable_path):
      return

    with open(self.unuploadable_path, encoding='utf-8', errors='ignore') as fh:
      for line in fh:
        s = line.strip()
        if not s:
          continue

        parts = s.split(",")
        pid = parts[0]
        self.unuploadable_products[pid] = parts[-1]

  def save_unuploadable_products(self):
    if not self.unuploadable_products:
      return

    records = []
    with open(self.unuploadable_path, 'w+', encoding='utf-8', errors='ignore') as fh:
      for pid, reason in self.unuploadable_products.items():
        records.append('{},{}'.format(pid, reason))

      fh.write("\n".join(records))
      fh.write("\n")
